from django import forms
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

from .settings import EMAIL_IS_USERNAME


class SocialUserCreationForm(forms.ModelForm):
    """
    Form that handles the validation of the social users setup.
    """
    username = forms.RegexField(
        required=not EMAIL_IS_USERNAME,
        max_length=30,
        regex=r'^[\w.@+-]+$',
        help_text=_('Required. 30 characters or fewer. Letters, digits and '
                    '@/./+/-/_ only.'),
        error_messages={'invalid': _('This value may contain only letters, '
                        'numbers and @/./+/-/_ characters.')},
        label=_('username').capitalize()
    )

    email = forms.EmailField(
        required=EMAIL_IS_USERNAME,
        max_length=75,
        label=_('email').capitalize()
    )

    class Meta:
        model = User
        fields = ['username', 'email', 'first_name', 'last_name']

    def __init__(self, *args, **kwargs):
        super(SocialUserCreationForm, self).__init__(*args, **kwargs)

        # If the email is used as username in the site, then do not show
        # this field on the form.
        if EMAIL_IS_USERNAME:
            del self.fields['username']

    def clean_username(self):
        """
        Verifies that the given username is not used by any already
        registered user in the site.
        """
        try:
            user = User.objects.get(username=self.cleaned_data['username'])

        except User.DoesNotExist:
            user = None

        if user is not None:
            raise forms.ValidationError(_(
                'This username is already in use, please provide a '
                'different username.'
            ))

        return self.cleaned_data['username']

    def clean_email(self):
        """
        Verifies that the given email address is not used by any already
        registered user in the site.
        """
        try:
            user = User.objects.get(email=self.cleaned_data['email'])

        except User.DoesNotExist:
            user = None

        if user is not None:
            raise forms.ValidationError(_(
                'This email address is already in use, please provide a '
                'different email address.'
            ))

        return self.cleaned_data['email']



from django.utils.translation import ugettext_lazy as _

from ..core.models import BaseOAuth1Profile


class TwitterOAuthProfile(BaseOAuth1Profile):
    """
    Model to store the Twitter OAuth related data.
    """
    class Meta:
        verbose_name = _('twitter profile')
        verbose_name_plural = _('twitter profiles')



from django.utils.translation import ugettext_lazy as _


# Default label for the only login buttons
DEFAULT_LOGIN_LABEL = _('Log in with %(service)s')

# Default label for the login/sigin buttons
DEFAULT_SIGNIN_LABEL = _('Sign in with %(service)s')

# Default message to show when the user is not registered using the only login
# button.
DEFAULT_ERROR_MESSAGE = _(
    'There is no account that matches your %(service)s credentials. You need '
    'to register with your %(service)s account before you can log in with '
    'its credentials.'
)



from .models import FacebookOAuthProfile
from ..core.backends import BaseSocialBackend


class FacebookBackend(BaseSocialBackend):
    """
    Backend to handle login with Facebook.
    """
    model = FacebookOAuthProfile



from .models import PayPalOAuthProfile
from ..core.backends import BaseSocialBackend


class PayPalBackend(BaseSocialBackend):
    """
    Backend to handle login with PayPal.
    """
    model = PayPalOAuthProfile



from django.dispatch import Signal


# Signal sent when an already registered but inactive user connects its
# social profile and it is actived by the app.
activation = Signal(providing_args=['user', 'service'])

# Signal sent when a user connects its account with a social profile.
connect = Signal(providing_args=['user', 'service'])

# Signal sent when a user disconnects its account from a social profile.
disconnect = Signal(providing_args=['user', 'service'])

# Signal sent when a user logs in with a social profile.
login = Signal(providing_args=['user', 'service'])



from django.utils.translation import ugettext_lazy as _

from ..core.models import BaseOAuth2Profile


class FacebookOAuthProfile(BaseOAuth2Profile):
    """
    Model to store the Facebook OAuth related data.
    """
    class Meta:
        verbose_name = _('facebook profile')
        verbose_name_plural = _('facebook profiles')



from hashlib import md5
from urllib import urlencode

from django import template


register = template.Library()


@register.simple_tag(takes_context=True)
def gravatar(context, user=None, email=None, gid=None, size=40, default=''):
    """
    Retrieves the gravatar url from a given user, email or hash.
    """
    gravatar_url = 'http://www.gravatar.com/avatar/'

    if ('user' in context and context['user'].is_authenticated() and
            not (user or email or gid)):
        _hash = md5(context['user'].email).hexdigest()

    elif user and not (email or gid):
        _hash = md5(user.email).hexdigest()

    elif email and not gid:
        _hash = md5(email).hexdigest()

    elif gid:
        _hash = gid

    else:
        _hash = None

    if _hash:
        return gravatar_url + _hash + '?' + urlencode(
            {'d': default, 's': size})

    return ''



from .models import GitHubOAuthProfile
from ..core.backends import BaseSocialBackend


class GitHubBackend(BaseSocialBackend):
    """
    Backend to handle login with GitHub.
    """
    model = GitHubOAuthProfile



from django.conf.urls import url, patterns

from . import views


urlpatterns = patterns(
    '',
    url(r'^login/$',
        views.GitHubDialogRedirect.as_view(), name='login'),
    url(r'^callback/$',
        views.GitHubCallback.as_view(), name='callback'),
    url(r'^setup/$',
        views.GitHubSetup.as_view(), name='setup'),
    url(r'^disconnect/$',
        views.GitHubOAuthDisconnect.as_view(), name='disconnect'),
)



from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _


class BaseSocialProfile(models.Model):
    """
    Base model that stores the user's social network information.
    """
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        blank=True, null=True,
        verbose_name=_('user')
    )
    service_uid = models.CharField(
        max_length=255,
        unique=True,
        blank=True, null=True,
        verbose_name=_('uid')
    )

    created_date = models.DateTimeField(
        blank=True, null=True,
        auto_now=False, auto_now_add=True,
        verbose_name=_('created date')
    )
    last_modified = models.DateTimeField(
        null=True, blank=True,
        auto_now_add=True, auto_now=True,
        verbose_name=_('last modified')
    )

    class Meta:
        abstract = True


class BaseOAuth1Profile(BaseSocialProfile):
    """
    Base Model that stores OAuth 1.0 flow information.
    """
    oauth_request_token = models.CharField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('OAuth request token')
    )
    oauth_request_token_secret = models.CharField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('OAuth request token secret')
    )
    oauth_access_token = models.CharField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('OAuth access token')
    )
    oauth_access_token_secret = models.CharField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('OAuth access token secret')
    )

    class Meta:
        abstract = True


class BaseOAuth2Profile(BaseSocialProfile):
    """
    Base Model that stores OAuth 2.0 flow information.
    """
    oauth_access_token = models.CharField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('OAuth access token')
    )
    oauth_access_token_expires_at = models.DateTimeField(
        blank=True, null=True,
        verbose_name=_('OAuth access token expires at')
    )

    class Meta:
        abstract = True



from .models import LinkedInOAuthProfile
from ..core.backends import BaseSocialBackend


class LinkedInBackend(BaseSocialBackend):
    """
    Backend to handle login with LinkedIn.
    """
    model = LinkedInOAuthProfile



from django.core.exceptions import ImproperlyConfigured

from ..core.settings import CONFIGURATION, COOKIE_MAX_AGE


# Tries to get the LINKEDIN configuration, if the configuration is not
# provided or is incorrect raises an ImproperlyConfigured exception.
LINKEDIN = CONFIGURATION.get('LINKEDIN', None)

if not LINKEDIN:
    raise ImproperlyConfigured(
        'You have socialnetworks.linkedin in your ISTALLED_APPS, but you do '
        'not specify any "LINKEDIN" settings inside '
        'SOCIALNETWORK_CONFIGURATION.'
    )

else:
    APP_ID = LINKEDIN.get('APP_ID', None)
    APP_SECRET = LINKEDIN.get('APP_SECRET', None)
    SESSION_KEY = LINKEDIN.get('SESSION_KEY', 'dsnli')
    SESSION_FIELDS = ','.join(LINKEDIN.get(
        'SESSION_FIELDS', ['firstName', 'lastName']))
    SETUP_URL_NAME = LINKEDIN.get('SETUP_URL_NAME', None)

    if not APP_ID:
        raise ImproperlyConfigured(
            'A proper "APP_ID" must be specified in order to use '
            'socialnetworks.linkedin module.'
        )

    if not APP_SECRET:
        raise ImproperlyConfigured(
            'A proper "APP_SECRET" must be specified in order to use '
            'socialnetworks.linkedin module.'
        )



from .clients import PayPalClient
from .settings import SETUP_URL_NAME
from ..core import views


class PayPalDialogRedirect(views.OAuthDialogRedirectView):
    """
    View that handles the redirects for the PayPal authorization dialog.
    """
    client_class = PayPalClient
#    def get_redirect_url(self):
#        return self.client.encode_url(self.client.authorization_url, {
#            'client_id': self.client.app_key,
#            'redirect_uri': self.get_callback_url(),
#            'response_type': 'code',
#            'scope': self.client.scope,
#            'nonce': uuid4()
#        })


class PayPalCallback(views.OAuthCallbackView):
    """
    View that handles the PayPal OAuth callback.
    """
    client_class = PayPalClient
#    def get_redirect_url(self):
#        if self.session_get('new_user'):
#            return reverse('socialnetworks:paypal:setup')
#
#        else:
#            return self.session_pop('next') or '/'


class PayPalSetup(views.OAuthSetupView):
    """
    View that handles the setup of a retrieved PayPal account.
    """
    client_class = PayPalClient
    setup_url = SETUP_URL_NAME
#    def retrieve_user_data(self):
#        # Retrieves the proper profile.
#        profile = self.get_profile()
#
#        # Creates a client that can make signed requests.
#        pp = PayPalClient(profile)
#
#        # Retrieve the user info from the api.
#        data = pp.get(pp.token_debug_url)
#
#        return {
#            'first_name': data['given_name'],
#            'last_name': data['family_name'],
#            'email': data['email']
#        }


class PayPalOAuthDisconnect(views.OAuthDisconnectView):
    """
    View that handles the disconnect of a PayPal account.
    """
    client_class = PayPalClient



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

from socialnetworks import __author__, __version__


setup(
    name='django-socialnetworks',
    version=__version__,
    description=('Extends Django with “log in” and “share” functionalities '
                 'for the most common social networks.'),
    long_description=open(
        os.path.join(os.path.dirname(__file__), 'README.rst')
    ).read(),
    author=__author__,
    author_email='zurg.cei@gmail.com',
    url='https://github.com/gGonz/django-socialnetworks',
    license='Apache License (2.0)',
    install_requires=[
        'Django >= 1.5',
        'requests',
        'requests_oauthlib',
        'unidecode',
    ],
    packages=find_packages(),
    include_package_data=True,
    classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Environment :: Web Environment',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet'
    ]
)
